function WbzChoice() {
    var _log;
    var _sm; //the serviceManager
    var _cf;
    var _context;

    var _xpath;
    var _parentModel;
    var _outputNode;
    var _model;
    var _view;
    var _resolver;

    var _viewConfig;
    var _createConfigs = new Array();
    var _insertXPath;

    var _selectNode;
    var _selectedIndex;
    var _firstIndex;
    var _firstModel;

    this.enableLogging = function(logger) {
        _log = logger;
    }

    this.service = function(serviceManager) {
        _sm = serviceManager;
        _resolver =  _sm.lookup("wobzilla.xpath");
        _cf = _sm.lookup("wobzilla.componentFactory");
    }

    this.contextualize = function(context) {
        _context = context;
    }

    this.configure = function(config) {
        _insertXPath = config.getAttribute("insertAt");
        _xpath = WBZ_attribute(config,"select",".");
        var configNodes = new Array();
        var childs = config.childNodes;
        for (var i =0 ;i<childs.length;i++) {
            if (childs[i].namespaceURI == WOBZILLA_NS) {
                if (childs[i].localName == "create") {
                    _createConfigs.push(childs[i]);
                    continue;
                } 
            }
            configNodes.push(childs[i]);
        }
        _viewConfig = WBZ_wrapChildConfigs(configNodes);
    }    

    this.init = function( outputNode, parentModel, insertAfter) {
        _parentModel = parentModel;
        _selectNode = document.createElementNS(HTML_NS,"select");
        _outputNode = document.createElementNS(HTML_NS,"div");
        _selectNode.setAttribute("class","choiceMenu");
        var containerNode = document.createElementNS(HTML_NS,"div");
        containerNode.setAttribute("class","choice");
        containerNode.appendChild( _selectNode );
        containerNode.appendChild( _outputNode );
        WBZ_insertAfter( outputNode, containerNode, insertAfter);
        insertAfter = containerNode;

        _selectNode.addEventListener("click",select,false);

        for (var i = 0; i < _createConfigs.length; i ++ ) {
            var option = document.createElementNS(HTML_NS,"option");
            var label = _createConfigs[i].getAttribute("label");
            var optionName = document.createTextNode(label);
            option.appendChild(optionName);
            option.setAttribute("value",i);
            _selectNode.appendChild(option);
        }

        _model = _resolver.getNode( _parentModel, _xpath, _context );
        _selectedIndex = findChoiceIndex( _model );
        if ( _selectedIndex >= 0 ) {
            _selectNode.value = _selectedIndex;
            _firstIndex = _selectedIndex;
            _firstModel = _model;
            initView();
        } 
        return insertAfter;
    }

    this.validate = function() {
        _view.validate();
    }

    this.dispose = function() {
        _cf.disposeComponent( _view );
    }

    function initView(){
        //dispose old view
        _cf.disposeComponent( _view ); 
        if ( _model == null ) 
            return;

        var context = new DefaultContext( _context );
        context.put( "wobzilla.position", new PositionContext(1,1) );
        // and replace with new
        _view = _cf.createComponent( _viewConfig, _log, context, _sm);
        _view.init( _outputNode, _model );
    }

    this.requery = function( changes ) {
        var newModel = _resolver.getNode( _parentModel, _xpath, _context );
        // Test if the model has changed
        if ( newModel != null && newModel.isSameNode( _model )) {
            _view.requery(changes);
            return;
        } else {
            if ( _model == newModel )
                 return;
            _model = newModel;
            WBZ_removeAll(_outputNode);
            _selectedIndex = findChoiceIndex( _model );
            _selectNode.value = _selectedIndex;
            initView();
        }
    }

    function findChoiceIndex( model) {
        if ( model == null ) 
            return -1;

        var elementName = model.localName;
        for (var i = 0 ;i < _createConfigs.length; i++ ) {
            var createElement = WBZ_getElements( _createConfigs[i] )[0];
            if (createElement.localName == elementName) {
                return i;
            }
        }
        throw new Error("Can't find choice for '" + elementName + "'");
    }

    function select(event) {
        try {
            if ( _selectedIndex != _selectNode.value ) {
                //WBZ_debug("select " + _selectNode.value);
                _selectedIndex = _selectNode.value;
                var newModel;
                if (_selectedIndex == _firstIndex) {
                    newModel = _firstModel;
                } else {
                    var config = _createConfigs[_selectedIndex];
                    newModel = WBZ_getFirstElement( config ).cloneNode(true);
                    //WBZ_setNewId(newModel,true);
                }
                _sm.lookup("wobzilla.transaction").start();
                if (_model != null)
                    _model.parentNode.removeChild(_model);
                insert(newModel);
                _sm.lookup("wobzilla.transaction").commit();
            }
        } catch (ex) {
            _log.error(ex);
        }
    }

    function insert( newModel ) {
        var insertPoint = getInsertPoint();
        if ( insertPoint != null ) {
            WBZ_insertAfter(insertPoint.parentNode, newModel, insertPoint);
        } else {
            WBZ_insertAfter(_parentModel, newModel, null);
        }
    }
   
    /** returns the last node that matches 
        the xpath expression from the "at" attribute belonging
        to the create configuration node.
     */
    function getInsertPoint() {
        if ( _insertXPath==null || _insertXPath=="" )
            return null;
        var childNodes = _resolver.getNodes( _parentModel, _insertXPath, _context );
        if ( childNodes.length == 0 ) {
            return null;
        } else {
            return childNodes[childNodes.length-1];
        }
    }

}
