function WbzContainer() {
    var _log;
    var _sm;
    var _cf;
    var _context;

    var _rootTemplate;
    var _childViews;
    var _outputNode;
    var _model;

    var _variableHandler;
    var _variableConfigs = null;
    var _resolver;
    var _cutDisabled;

    // The following variables only get used if a select attribute is set.
    var _xpath;
    var _markerStartNode;
    var _markerEndNode;
    var _parentModel;



    this.enableLogging = function(logger) {
        _log = logger;
    }

    this.contextualize = function(context) {
        _context = context;
        _context = new DefaultContext(_context);
        _cutDisabled = _context.get("wobzilla.defaultCut") != true;
    }

    this.service = function(serviceManager) {
        _sm = serviceManager;
        _cf = _sm.lookup("wobzilla.componentFactory");
        _resolver = _sm.lookup("wobzilla.xpath");
    }

    this.configure = function(config) {
        _rootTemplate = config;
        _xpath = config.getAttribute("select");

        // init variables and parameter values
        var childs = config.childNodes;
        for (var i = 0 ; i < childs.length; i++ ) {
            var child = childs[i];
            if (child.namespaceURI == WOBZILLA_NS
                && (child.localName == "param" || child.localName == "variable" )
                )
            {
                if ( _variableConfigs == null)
                    _variableConfigs = new Array();
                _variableConfigs.push(child);
            }
        }
        if (_xpath != null || _variableConfigs != null) {
            _context = new DefaultContext( _context );
            if ( _xpath != null ) {
                _context.put( "wobzilla.position", new PositionContext(1,1) );
            }
            if ( _variableConfigs != null ) {
                var localVariables = new DefaultContext(_context.get("wobzilla.local"));
                _context.put("wobzilla.local",localVariables);
                _context.put("wobzilla.localempty",false);
            }
        }
    }

    this.init = function(outputNode,parentModel,insertAfter) {
        _outputNode = outputNode;
        if ( _xpath ) {
            if ( insertAfter != null ) {
                _markerStartNode = document.createTextNode("");
                WBZ_insertAfter( _outputNode, _markerStartNode, insertAfter );
                insertAfter = _markerStartNode;
            } 
            _markerEndNode = document.createTextNode("");
            WBZ_insertAfter( _outputNode, _markerEndNode, insertAfter);

            _parentModel = parentModel;
            _model = _resolver.getNodeSafe( _parentModel, _xpath, _context);
        } else {
            _model = parentModel;
        }
        return myinit(insertAfter);
    }

    // validates the child views
    this.validate = function() {
        for (var i = 0; i < _childViews.length; i++) {
            _childViews[i].validate();
        }
    }

    this.dispose = function() {
        disposeChilds();
    }

    function disposeChilds() {
        if ( _childViews != null ) {
            for (var i=0; i < _childViews.length; i++) {
                _cf.disposeComponent( _childViews[i] );
            }
        }
    }

    function myinit(insertAfter) {
        // create und initialize variables;
        if ( _variableConfigs != null) {
            var fact = _sm.lookup("wobzilla.variableFactory");
            _variableHandler = fact.createVariableHandler( _variableConfigs, _log, _context, _sm);
            var localVariables = _context.get("wobzilla.local");
            _variableHandler.init(_model,localVariables);
        } 
        disposeChilds();
        _childViews = new Array();
        var childs = _rootTemplate.childNodes;
        for (var i=0;i<childs.length;i++) {
            insertAfter = processTemplateNode(_outputNode,childs[i],insertAfter);
        }
        return insertAfter;
    }

    function processTemplateNode( outputNode, configNode, insertAfter) {
        // createChildView
        if (configNode.namespaceURI == WOBZILLA_NS) {
            if (configNode.localName == "variable" || configNode.localName == "param") {
                return insertAfter;
            } else {
                var childView = _cf.createComponent( configNode, _log, _context, _sm );
                _childViews.push( childView );
                return childView.init( outputNode, _model, insertAfter );
            }
        } else {
            var newChild = configNode.cloneNode(false);
            WBZ_insertAfter( outputNode, newChild, insertAfter);
            insertAfter = newChild;
            var nodes = configNode.childNodes;
            var childInsert = null;
            for (var i=0;i<nodes.length;i++) {
                childInsert = processTemplateNode( newChild, nodes[i], childInsert );
            }
            return insertAfter
        }
    }
    
    this.requery = function( changes ) {
        //_log.info("processing changes for " + changes.target.nodeName);
        if ( _xpath ) {
            var newModel = _resolver.getNodeSafe( _parentModel, _xpath, _context );
            if ( !newModel.isSameNode( _model ) ) {
                _model = newModel;
                rebuild();
                return;
            }
        }

        //_log.info(changes.localVariableChanged + " " + changes.paramChanged);

        // Cut all events that are not targeted to childrens of the model
        // or the model itself
        if ( ! _cutDisabled && !WBZ_isTargeted( changes, _model ) ) {
            return;
        }

        // test if variable have changed
        var variableChanged = false;
        if ( _variableHandler != null ) {
            variableChanged = _variableHandler.requery(changes);
        }
        // and create a new changes event if neccessary
        if ( variableChanged && changes.localVariableChanged != true )
        {
            changes = WBZ_clone(changes);
            // set new localVariable flag
            changes.localVariableChanged = true;
        }

        for (var i = 0 ;i < _childViews.length; i++) {
            _childViews[i].requery(changes);
        }
    }

    function rebuild() {
        if ( _log.debugging )
            _log.debug("Rebuilding Container");

        for (var i = 0; i < childViews.length; i++) {
            _cf.disposeComponent( _childViews[i] );
        }

        var currentNode = ( _markerStartNode != null ) ? 
            _markerStartNode.nextSibling : _outputNode.firstChild;
        while ( currentNode != null && 
                !currentNode.isSameNode( _markerEndNode ))
        {
            var removeNode = currentNode;
            currentNode = currentNode.nextSibling; 
            _outputNode.removeChild( removeNode );
        }
        myinit( _markerStartNode );
    }
}
