function WbzMapper() {
    var _cf;//componentFactory
    var _log;
    var _rootResolver;
    var _imported = new Array();
    var _matchingRules = new Array();
    var _namedRules = new Array();

    var _globalVariables = new DefaultContext();
    var _variableHandler;

    var _sm;
    var _context;
    var _config;

    var _rootView;
    
    this.enableLogging = function(logger) {
        _log = logger;
    }

    this.contextualize = function(context) {
        _context = new DefaultContext(context);
        _context.put("wobzilla.local",new DefaultContext());
        _context.put("wobzilla.arguments",context);
        _context.put("wobzilla.global",_globalVariables);
        _context.put("wobzilla.position", new PositionContext(1,1));
        _context.put("wobzilla.localempty",true);
    }

    this.service = function(serviceManager) {
        _sm = new DefaultServiceManager(serviceManager);
        _cf = _sm.lookup("wobzilla.componentFactory");
        _sm.put("wobzilla.viewFactory", this);
        _sm.put("wobzilla.variableFactory", this);
        _sm.put("wobzilla.sortFactory", this);
    }

    this.configure = function(config) {
        _config = config;
        _rootResolver = this.createResolver();
        var resolver = new ContextXPathResolver( _rootResolver, _globalVariables);
        _cf.setup(resolver,null,_log,_context,_sm);
        _sm.put("wobzilla.xpath", resolver);

        _context.put("wobzilla.defaultCut", config.getAttribute("default-cut") == "true");
        var childs = config.childNodes;
        var variableConfigs = null;
        for (var i = 0;i<childs.length; i++) { 
            var child = childs[i];
            if (child.namespaceURI != WOBZILLA_NS) {
                continue;
            }
            switch (child.localName) {
            case "import": configureImport(childs[i]);break;
            case "variable": 
            case "param": 
                if (variableConfigs == null)
                    variableConfigs = new Array();
                variableConfigs.push(child);
                break;
            case "template": configureTemplate(childs[i]);break;
            default: throw new Error("Instruction " + childs[i].localName + " not yet supported");
            }
        }

        if ( variableConfigs != null) {
            _variableHandler =  this.createVariableHandler( 
                                                           variableConfigs
                                                           , _log
                                                           , _context
                                                           , _sm
                                                           );
        }
    }

    function configureImport(config) {
        var href = config.getAttribute("href");
        if (href == null)
            throw new Error("Missing href attribute!");
        _log.debug("Importing " + href);
        var id = config.getAttribute("id");
        var importLogger = (id != null) ? _log.getChildLogger(id): _log;
        
        importConfig = WBZ_getFirstElement( _sm.lookup("wobzilla.xmlLoader").load(href));
        var importedMapper = _cf.createComponent(
                                                importConfig
                                                ,importLogger
                                                ,_context
                                                ,_sm);
        _imported.push(importedMapper);
    }

    function configureTemplate(config) {
        var rule = new Object();
        rule.mode = config.getAttribute("mode");
        rule.name = config.getAttribute("name");
        var match = config.getAttribute("match");
        if ( match != null) {
            rule.match = _rootResolver.createExpression(match);
        } else if (rule.name == null) {
            throw new Error("Template element needs match attribute.");
        }
        rule.view = config;
        if (rule.name != null)
            _namedRules.push(rule);
        else
            _matchingRules.push(rule);
    }

    this.init = function(outputNode,modelNode,insertAfter) {
        // init variables and parameter values
        if (_variableHandler != null) {
            _variableHandler.init( modelNode, _globalVariables );
        }

        // init root-view
        _rootView = this.createView( modelNode, null, _log, _context, _sm);
        _rootView.init( outputNode, modelNode, insertAfter);

        if ( _rootResolver.isProfiling() ) {
            _log.info("XPATH time " + _rootResolver.getProfileTime() + " milliseconds");
        }

    }

    this.requery = function(changes) {
        // test if variable have changed
        var variableChanged;
        if ( _variableHandler != null ) {
            variableChanged = _variableHandler.requery(changes);
        }

        // and create a new changes event if neccessary
        if (variableChanged && changes.globalVariableChanged != true)
        {
            newChanges = new Object();
            // clone changes object;
            for (var id in changes) {
                newChanges[id] = changes[id];
            }
            // set new globalVariable flag
            newChanges.globalVariableChanged = true;
            changes = newChanges;
        }

        return _rootView.requery(changes);
    }    

    this.validate = function() {
        _rootView.validate();
        if ( _variableHandler != null ) {
            return _variableHandler.validate();
        } else {
            return true;
        }
    }

    this.findRule = function(contextNode,mode,importedRulesOnly,templateName) {
        if (importedRulesOnly == null || importedRulesOnly == false ) {
            // search named template rules
            if (templateName != null) {
                for (var i = 0; i < _namedRules.length; i++) {
                    var rule = _namedRules[i];
                    if (rule.name == templateName) 
                        return WBZ_wrapConfig(rule.view);
                    else
                        continue;
                } 
            }
            // search matching template rules
            for (var i = 0; i < _matchingRules.length; i++) {
                var rule = _matchingRules[i];
                if ( mode == rule.mode && xpathMatch(contextNode,rule.match ) )
                    return WBZ_wrapConfig( rule.view );
            }
        }
        for (var i = 0;i< _imported.length; i++) {
            var rule = _imported[i];
            var viewConfig = rule.findRule(contextNode,mode,false,templateName);
            if (viewConfig != null )
                return viewConfig;
        }
        return null;
    }

    this.createView = function(contextNode
                               ,mode
                               ,logger
                               ,context
                               ,serviceManager
                               ,importedRulesOnly
                               ,templateName
                               ,args
                               ) {
        if (_log.debugging) {
            _log.debug("Creating Component for '" + contextNode.nodeName + "'");
        }
        
        var viewConfig = this.findRule(contextNode,mode,importedRulesOnly,templateName);
        if (viewConfig == null) {
            throw new Error("No template rule for '" + contextNode.nodeName + "'");
        }
        if (context.get("wobzilla.localempty") == true || args != null) {
            serviceManager = new DefaultServiceManager(serviceManager);
            context = new DefaultContext(context);
            if (args == null) {
                args = new DefaultContext();
            }
            context.put("wobzilla.arguments",args);
            context.put("wobzilla.localempty",true);
            context.put("wobzilla.local",new DefaultContext());
        }
        return _cf.createComponent(viewConfig,logger,context,serviceManager);
    }

    this.dispose = function () {
        this.disposeComponent( _rootView );
    }

    this.disposeComponent = function ( component ) {
        _cf.disposeComponent( component );
    }

    this.createResolver = function() {
        var resolver = new DefaultXPathResolver( _config.ownerDocument, _config);
        _cf.setup(resolver, null, _log.getChildLogger("xpath"), _context, _sm);
        return resolver;
    }

    this.createVariableHandler = function( variableConfigs, logger, context, sm) {
        var variableHandler = new DefaultVariableHandler( variableConfigs , onvariablechange);
        _cf.setup( variableHandler, null, logger, context, sm);
        return variableHandler;
    }

    this.createSortHandler = function( sortConfigs, logger, context, sm) {
        var sortHandler = new DefaultSortHandler( sortConfigs );
        _cf.setup( sortHandler, null, logger, context, sm);
        return sortHandler;
    }
    
    function xpathMatch(node, rule) {
        var documentNode;
        if ( node.ownerDocument != null) {
            documentNode = node.ownerDocument;
        } else {
            // #TODO check if node is document
            documentNode = node;
        }
        var currentNode = node;
        while (true) {
            if (currentNode == null)
                throw new Error("'" + node.nodeName + "' node not in xml-document.");
            if ( _rootResolver.matches( currentNode, rule, node))
                return true;
            if ( currentNode.isSameNode( documentNode)) {
                return false;
            } else {
                switch (currentNode.nodeType) {
                case ELEMENT_NODE:  currentNode = currentNode.parentNode; break;
                case ATTRIBUTE_NODE: currentNode = currentNode.ownerElement;break;
                default: throw new Error("Can't match node-type "+ currentNode.nodeType + " yet");
                }
            }
            
        }
        return false;
    }

    function onvariablechange() {
        var changes = new Object();
        changes.events = new Array();
        changes.globalVariableChanged = true;
        _rootView.requery( changes );
    }
}


/**
  @implements wobzilla_variableHandler
  @constructor
*/
function DefaultVariableHandler( _variableConfigs, _onvariablechange) {
    var _variables = new Array();
    var _outputContext;
    var _sm;
    var _context;
    var _log;
    var _resolver;

    this.enableLogging = function(logger) {
        _log = logger;
    }

    this.contextualize = function( context ) {
        _context = context;
    }

    this.service = function(serviceManager) {
        _sm = serviceManager;
        _resolver = _sm.lookup("wobzilla.xpath");
        for ( var i=0; i < _variableConfigs.length; i++) {
            addVariable( _variableConfigs[i] );
        }
    }

    this.init = function( modelNode, outputContext) {
        _outputContext = outputContext;
        for (var i = 0; i< _variables.length; i++) {
            var variable = _variables[i];
            // look if param value is on context
            var providedParam = null;
            // look for provided parameters
            if ( _context.has( "wobzilla.arguments" ) ) {
                var args = _context.get( "wobzilla.arguments" );
                if ( args.has( variable.name ) )
                    providedParam = args.get( variable.name );
            }
            if ( variable.type == "param" && providedParam ) {
                variable.paramUsed = true;
                variable.content = providedParam;
            } else if ( variable.xpath == null ) {
                var outputNode = document.createElementNS( WOBZILLA_NS, "dummy");
                variable.view.init( outputNode, modelNode, null );
                variable.output = outputNode;
            } else {
                variable.model = modelNode;
            }
            if ( variable.onvalidate ) {
                _outputContext.put( variable.name, "true()" );
            } else {
                calculate( variable );
            }
        }
    }

    this.requery = function( changes ) {
        var variableChanged = false;
        for ( var i=0; i < _variables.length; i++ ) {
            var variable = _variables[i];
            // don't test parameters, changes are handled by the caller
            if ( variable.paramUsed || variable.onvalidate )
                continue;

            if ( variable.view != null ) {
                variable.view.requery( changes );
            }
            if ( calculate( variable ) ) {
                variableChanged = true;
            }
        }
        return variableChanged;
    }

    this.validate = function() {
        var valid = true;
        var variableChanged = false;
        for ( var i=0; i < _variables.length; i++ ) {
            var variable = _variables[i];
            if ( variable.onvalidate ) {
                // recalculate variable
                if ( variable.view != null ) {
                    var changes = new Object();
                    changes.events = new Array();
                    // this flag will trigger a complete rebuild of the variable
                    changes.globalVariableChanged = true; 
                    variable.view.requery( changes );
                }
                if ( calculate( variable ) ) {
                    variableChanged= true;
                }
            }
            if ( variable.validation ) {
                if ( variable.content == "false()" )
                    valid = false;
            }
        }
        if ( variableChanged && _onvariablechange) {
            _onvariablechange();
        }
        return valid;
    }

    function addVariable( config ) {
        var variable = new Object();
        variable.type = config.localName;
        variable.name = config.getAttribute( "name" );
        variable.xpath = config.getAttribute( "select" );
        variable.onvalidate = config.getAttribute( "evaluate" ) == "onvalidate";
        variable.validation = config.getAttribute( "validation" ) == "true";
        if (variable.xpath == null) {
            var fact = _sm.lookup("wobzilla.componentFactory");
            variable.view = fact.createComponent(
                                                 WBZ_wrapConfig(config)
                                                 ,_log
                                                 ,_context
                                                 ,_sm
                                                 );
        }
        _variables.push( variable );
    }

    /** calculates the text-content of a variable. Returns true if
     * content has changed and false otherwise. */
    function calculate( variable ) {
        var content = "";
        //_log.info("calculate " + variable.name);
        if ( variable.paramUsed ) {
            content = variable.content;
        } else if ( variable.xpath != null ) {
            content = evalXPath( variable.model, variable.xpath );
        } else {
            var childs = variable.output.childNodes;
            if ( childs.length == 1 ) {
                content = childs[0].nodeValue;
            } else {
                for ( var i=0; i < childs.length; i++ ) {
                    content += childs[i].nodeValue;
                }
            }
            content = encode( content );
        }
        if ( _log.debugging )
            _log.debug( "Setting variable " + variable.name  + " = " + content );

        _outputContext.put( variable.name, content );

        if ( variable.content != content ) {
            variable.content = content;
            return true;
        } else {
            return false;
        }
    }

    function evalXPath( model, xpath ) {
        var result = _resolver.getAnyType( model, xpath, _context);
        switch ( result.resultType ) {
        case XPATH_NUMBER_TYPE: 
            //_log.info("NUMBER");
            return result.numberValue;
        case XPATH_BOOLEAN_TYPE: 
            return result.booleanValue + "()";
        case XPATH_STRING_TYPE: 
            //_log.info("STRING");
            return encode( result.stringValue );
        case XPATH_UNORDERED_NODE_ITERATOR_TYPE: 
        case XPATH_ORDERED_NODE_ITERATOR_TYPE: 
            //_log.info("STRING");
            var node = result.iterateNext();
            if ( node == null )
                return "''";
            var content = "";
            var childs = node.childNodes;
            for ( var i=0; i < childs.length; i++ ) {
                content += childs[i].nodeValue;
            }
            return encode( content );
        case XPATH_ANY_UNORDERED_NODE_TYPE: 
        case XPATH_FIRST_ORDERER_NODE_TYPE: 
            return encode( result.singleNodeValue );
        }
    }


    function encode( string ) {
        if ( string == null )
            return "''";
        // Test if we must encode the quotes
        if (string.indexOf("\"") <0) {
            return "\"" + string + "\"";
        } else {
            // We can't use &quot; here so we must use the concat function
            var elements = string.split("\"");
            var result = "concat(\"";
            for (var i=0;i<elements.length-1;i++) {
                result += elements[i];
                result += "\",'\"',\"";
            }
            result += elements[elements.length-1];
            result += "\")";
            return result;
        }
    }
}

/**
This sortHandler uses the java-script Array.sort Method for sorting.
It currently supports english and german collating rules.
 @implements wobzilla_sortHandler
 @constructor
*/
function DefaultSortHandler( _sortConfigs ) {
    var _resolver;
    var _log;
    var _sortRules  = new Array( _sortConfigs.length );

    this.enableLogging = function(logger) {
        _log = logger;
    }

    this.contextualize = function( context ) {
        _context = context;
    }

    this.service = function(serviceManager) {
        _resolver = serviceManager.lookup("wobzilla.xpath");

        for (var i=0; i < _sortConfigs.length; i++) {
            var config = _sortConfigs[i];
            var xpath = WBZ_attribute(config, "select");
            _sortRules[i] = new Object();
            _sortRules[i].xpath = xpath;
            _sortRules[i].lang = WBZ_attribute(config, "lang", "en");
            _sortRules[i].descending = WBZ_attribute(config, "order") == "descending";
            _sortRules[i].numeric = WBZ_attribute(config,"data-type") == "number";
        }
    }

    this.sort = function( entries, relocateMethod ) {

        // We need to rebuild the sort-expression before sorting
        // (they might contain variables)
        for (var i=0; i < _sortRules.length; i++) {
            _sortRules[i].expression = _resolver.createExpression(_sortRules[i].xpath);
        }

        var keysChanged = false;
        var positionsChanged = false;
        for (var i = 0;i < entries.length; i++) {
            if (calculateKeys( entries[i] )) {
                keysChanged = true;
            }
        }
        if (keysChanged) {
            var sortedArray = new Array();
            for (var i=0; i < entries.length; i++) {
                entries[i].positionContext.documentPosition = i + 1;
                sortedArray.push( entries[i] );
            }
            sortedArray = sortedArray.sort(compareEntries);
            for (var i=sortedArray.length-1;i>=0; i--) {
                var entry = sortedArray[i];
                var following = (i<sortedArray.length) ? sortedArray[i+1] : null;
                //_log.info(i + ": " + WBZ_serializeXML(entry.model) )
                if (entry.nextEntry != following) {
                    entry.nextEntry = following ;
                    relocateMethod( entry );
                    positionsChanged = true;
                }
                entry.positionContext.position = i + 1;
                entry.positionContext.last = entries.length;
            }
        }
        return positionsChanged;
    }

    /** calculates new keys and returns if keys have changed*/
    function calculateKeys(entry) {
        if (entry.sortKeys == null) {
            entry.sortKeys = new Array(_sortRules.length);
        }
        var keys = entry.sortKeys;
        var invalid = false;
        for (var i=0;i<keys.length;i++) {
            var newKey = evaluate(i,entry.model);
            //_log.info(newKey +" " + keys[i]);
            if (newKey != keys[i]) {
                keys[i] = newKey;
                invalid = true;
            }
        }
        return invalid;
    }

    function evaluate(ruleIndex,model) {
        var expression = _sortRules[ruleIndex].expression;
        if (_sortRules[ruleIndex].numeric) {
            return expression.evaluate(model
                                       ,XPATH_NUMBER_TYPE
                                       ,null
                                       ).numberValue;
        } else {
            var s = expression.evaluate(model
                                        ,XPATH_STRING_TYPE
                                        ,null
                                        ).stringValue.toLowerCase();
            // at the moment, only german collating rules are implemented
            if (_sortRules[ruleIndex].lang == "de") {
                s = s.replace(//g,"a");
                s = s.replace(//g,"o");
                s = s.replace(//g,"u");
                s = s.replace(//g,"s");
            }
            return s;
        }
    }

    function compareEntries(firstEntry,secondEntry) {
        //_log.info("comparing " + WBZ_serializeXML(firstEntry.model) 
        //+ " with "  + WBZ_serializeXML(secondEntry.model) );        
        if (firstEntry == secondEntry)
            return 0;
        var keys = firstEntry.sortKeys;
        var otherKeys = secondEntry.sortKeys;
        // test key position
        for (var i=0;i<otherKeys.length;i++) {
            if (keys[i]>otherKeys[i]) {
                //                _log.info("A > B");
                return (_sortRules[i].descending) ? -1 : 1;
            } else if (keys[i]<otherKeys[i]) {
                //  _log.info("A < B");
                return (_sortRules[i].descending) ? 1 : -1;
            }
        }
        // test document position
        if (firstEntry.position>secondEntry.position) {
            //  _log.info("A > B position ");
            return 1;
        } else {
            //_log.info("A < B position ");
            return -1;
        }
    }
}


var XPATH_ANY_TYPE = 0;
var XPATH_NUMBER_TYPE = 1;
var XPATH_STRING_TYPE = 2;
var XPATH_BOOLEAN_TYPE = 3;
var XPATH_UNORDERED_NODE_ITERATOR_TYPE = 4;
var XPATH_ORDERED_NODE_ITERATOR_TYPE = 5;
var XPATH_UNORDERED_NODE_SNAPSHOT_TYPE = 6;
var XPATH_ORDERED_NODE_SNAPSHOT_TYPE = 7;
var XPATH_ANY_UNORDERED_NODE_TYPE = 8;
var XPATH_FIRST_ORDERED_NODE_TYPE = 9;

function DefaultXPathResolver(document,root) {
    var doc = document;
    //    var root = WBZ_getFirstElement(doc);
    var namespaces = doc.createNSResolver(root);
    var profile = false;
    var profileTime;
    var startTime = 0; 
    var endTime;
    var log;

    this.enableLogging = function(logger) {
        log = logger;
    }

    this.contextualize = function(context) {
        if (context.has("wobzilla_xpath_profile"))
            this.setProfiling(context.get("wobzilla_xpath_profile") == "true()");
    }

    this.setProfiling = function(enabled) {
        profile = enabled;
        profileTime = 0;
    }

    this.isProfiling = function() {
        return profile;
    }

    this.getProfileTime = function() {
        return profileTime;
    }

    this.getNodeSafe = function(modelNode, xpath) {
        var node = this.getNode(modelNode,xpath);
        if (node == null) {
            throw new Error("Xpath-Expression '" + xpath + "' empty in context " + modelNode.nodeName);
        }
        return node;
    }

    this.getNode = function(modelNode, xpath) {
        check(modelNode,xpath);
        if (xpath == ".") 
            return modelNode;

        if (profile) startProfile();
        var result = doc.evaluate(xpath
                                  ,modelNode
                                  ,namespaces
                                  ,XPATH_FIRST_ORDERED_NODE_TYPE
                                  ,null
                                  // Don't reuse result here or mozilla
                                  // will crash
                              );
        if (profile) endProfile();
        return result.singleNodeValue;
    }


    this.getNumber = function(modelNode, xpath) {
        check(modelNode,xpath);
        if (profile) startProfile();
        var result = doc.evaluate(xpath
                                  ,modelNode
                                  ,namespaces
                                  ,XPATH_NUMBER_TYPE
                                  ,null
                                  );
        if (profile) endProfile();
        return result.numberValue;
    }

    function check(modelNode,xpath) {
        if (xpath == null) {
           throw new Error("Need to specify an xpath argument.");
        }
        if (modelNode == null) {
           throw new Error("Error: context node can't be null!");
        }
        //log.debug(" Selecting: '" + xpath + "' in context '" + modelNode.nodeName + "'");
    }
    this.getText = function(modelNode, xpath) {
        check(modelNode,xpath);
        if (profile) startProfile();
        var result = doc.evaluate(xpath
                                  ,modelNode
                                  ,namespaces
                                  ,XPATH_STRING_TYPE
                                  ,null
                                  );
        if (profile) endProfile();
        return result.stringValue;
    }

    this.getBoolean = function(modelNode, xpath) {
        if (profile) startProfile();
        check(modelNode,xpath);
        var result = doc.evaluate(xpath
                                  ,modelNode
                                  ,namespaces
                                  ,XPATH_BOOLEAN_TYPE
                                  ,null
                                  );
        if (profile) endProfile();
        return result.booleanValue;
    }

    this.getAnyType = function(modelNode, xpath) {
        check(modelNode,xpath);
        if (profile) startProfile();

        var result = doc.evaluate(xpath
                                  ,modelNode
                                  ,namespaces
                                  ,XPATH_ANY_TYPE
                                  ,null
                                  );
        if (profile) endProfile();
        return result;
    }

    this.getNodes = function(modelNode, xpath) {
        check(modelNode,xpath);
        if (profile) startProfile();

        var resultSet = new Array();
        if (xpath == ".") {
            resultSet.push(modelNode);
            if (profile) endProfile();
            return resultSet;
        }
        var result = doc.evaluate(xpath
                                  ,modelNode
                                  ,namespaces
                                  ,XPATH_ORDERED_NODE_ITERATOR_TYPE
                                  ,null
                                  );
        var nextNode = result.iterateNext();
        while (nextNode) {
            resultSet.push(nextNode);
            nextNode = result.iterateNext();
        }
        if (profile) endProfile();
        return resultSet;
    }

    /** Create an XPath Expression object from a string*/
    this.createExpression = function(xpath) {
        return doc.createExpression(xpath,namespaces);
    }
        
    this.matches = function(contextNode,expression,node) {
        if (profile) startProfile();
        var result = expression.evaluate(contextNode
                                         ,XPATH_UNORDERED_NODE_ITERATOR_TYPE
                                         ,null
                                         );
        
        var nextNode = result.iterateNext();
        while (nextNode) {
            if (nextNode.isSameNode( node )) {
                if (profile) endProfile();
                return true;
            }
            nextNode = result.iterateNext();
        }
        if (profile) endProfile();
        return false;
    }

    function startProfile() {
        startTime = new Date().getTime(); 
    }

    function endProfile() {
        profileTime += (new Date()).getTime() - startTime;
    }


}

/** PositionContext
   @constructor
*/
function PositionContext(position,last) {
    this.documentPosition = position;
    this.position = position;
    this.last = last;
}


/**
   This implementation replaces all variable-values in xpath
   expressions with a correspondig entry in contex before delegating
   the call to the parent-Resolver. It also replaces all occurces of
   <code>position()</code> and <code>last()</code> with the values
   provided by the context entry in <code>wobzilla.position</code>.

   <p>Mozilla does not provide a way to specify variables or position functions 
   in xpath expression (At least not through the exposed apis). This class
   works around this by replacing the variable-names with the variable-values before
   further processing the xpath expression.
   </p>
   <p>
   This method currently replaces all occurences of $ followed by a word,
   even if the expression is quoted e.g. '$test'. To use $ in the output you have
   to use the concat function. Write concat('$','test') instead of '$test'.
   </p>

  @implements wobzilla_xpath
  @constructor
*/
function ContextXPathResolver(parent,global) {
    var log;

    this.enableLogging = function(logger) {
        log = logger;
    }
    
    this.getNodeSafe = function(modelNode, xpath, context) {
        return parent.getNodeSafe(modelNode, replace(xpath), context);
    }

    this.getNode = function(modelNode, xpath, context) {
        return parent.getNode(modelNode, replace(xpath, context), context);
    }

    this.getNumber = function(modelNode, xpath, context) {
        return parent.getNumber(modelNode, replace(xpath, context), context);
    }

    this.getText = function(modelNode, xpath, context) {
        return parent.getText(modelNode, replace(xpath, context), context);
    }

    this.getBoolean = function(modelNode, xpath, context) {
        return parent.getBoolean(modelNode, replace(xpath, context), context);
    }

    this.getNodes = function(modelNode, xpath, context) {
        return parent.getNodes(modelNode, replace(xpath, context), context);
    }

    this.getAnyType = function(modelNode, xpath, context) {
        return parent.getAnyType(modelNode, replace(xpath, context), context);
    }

    this.createExpression = function(xpath, context) {
        return parent.createExpression(xpath, replace(xpath, context), context);
    }

    function replace(xpath, context) {
        if (xpath == null)
            return xpath;
        xpath = replaceVariables(xpath, context);
        xpath = replaceContextFunctions(xpath, context);
        return xpath;
    }    

    /** replaces variables with text.
     */
    function replaceVariables(xpath,context) {
        if (xpath.indexOf('$') == -1)
            return xpath;
        var result = xpath.match(/\$\w[\w-]*/g);
        if (result == null)
            return xpath;

        var local = context.get("wobzilla.local");
        var args = context.get("wobzilla.arguments"); // can't use arguments, this is a reserved word in java-script

        for(var i=0;i<result.length;i++) {
            var variable = result[i];
            var key = variable.substr(1);
            var value;
            if (args.has(key)) {
                value = args.get(key);
            } else if (local.has(key)) {
                value = local.get(key);
            } else {
                value = global.get(key);
            }
            if (log.debugging)
                log.debug("Var " + variable + " value " + value);
            //regExp = new RegExp("\\" + variable);
            xpath = xpath.replace(variable, value);
        }
        //log.debug("New xpath " + xpath);
        return xpath;
    }

    /** replaces position() and last() functions if 
        they are used inside a top level expression (no []-brackets)
     */
    function replaceContextFunctions(xpath, context) {
        var index1 = xpath.indexOf('position()');
        var index2 = xpath.indexOf('last()');
        if ( index1 == -1 &&  index2== -1)
            return xpath;
        
        var string = xpath;
        var length;
        var index;
        if (index1>-1 && (index2 <0 || index2>index1)) {
            // position() is first
            length = 10;
            index = index1;
        } else {
            // last() is first
            length = 6;
            index = index2;
        }
        var openBrackets = 0;
        var positionContext = null;
        var i = 0;
        while (index > -1) {
            while (i<index) {
                var c = string.charAt(i);
                if (c == '[') {
                    openBrackets ++;
                    continue;
                }
                if (c == ']') {
                    openBrackets ++;
                    continue;
                }
                i++;
            }
            if (openBrackets == 0) {
                var before = string.slice(0,index);
                var after = string.slice(index + length);
                if (positionContext == null) {
                    positionContext = context.get("wobzilla.position");
                }
                
                var middle = (length == 6) ? 
                    positionContext.last : positionContext.position ;
                string = before + middle + after;
            }
            index1 = string.indexOf('position()',index + 1);                
            index2 = string.indexOf('last()',index + 1);
            if (index1>-1 && (index2 <0 || index2>index1)) {
                // position() is first
                length = 10;
                index = index1;
            } else {
                // last() is first
                length = 6;
                index = index2;
            }
            //log.info(string + " " + index + " " + index1 + " " + index2);
        }
        //        log.info("Replaced " + xpath + " with " + string);
        return string;
    }
}

