var FOR_EACH=0;
var APPLY_TEMPLATES=1;
var APPLY_IMPORTS=2;
var CALL_TEMPLATE=3;

function WbzApply() {
    var _sm;
    var _log;
    var _context;

    var _type;
    var _mode;
    var _args;
    var _templateName;
    var _importsOnly;

    var _variableHandler;
    var _sortHandler;
    var _childConfig;

    var _xpath;
    var _parentModel;

    var _entries = new Array();
        
    var _outputNode;

    var _markerStartNode;
    var _markerEndNode;
    var _resolver;
    
    var _cutDisabled;

    var _factory;

    this.enableLogging = function(logger) {
        _log = logger;
    }

    this.service = function(serviceManager) {
        _sm = serviceManager;
        _resolver = _sm.lookup("wobzilla.xpath");
    }

    this.contextualize = function(context) {
        _context = context;
        _cutDisabled = context.get("wobzilla.defaultCut") != true;
    }

    this.configure = function(config) {
        _xpath = WBZ_attribute(config, "select", (_type == CALL_TEMPLATE) ? "." : "*");
        switch (config.localName) {
        case "apply-templates" : 
            _type = APPLY_TEMPLATES;
            _mode = config.getAttribute("mode");
            break;
        case "apply-imports" : 
            _type = APPLY_IMPORTS;
            _importsOnly = true;
            _mode = config.getAttribute("mode");
            break;
        case "call-template" : 
            _type = CALL_TEMPLATE;
            _templateName = config.getAttribute("name");
            break;
        case "for-each" : 
            _type = FOR_EACH;
            break;
        default: 
            throw new Error("Invalid config-name: " + config.localName);
        }
        
        var childs = config.childNodes;
        var sortConfigs;
        var childConfigs;
        var variableConfigs = null;
        for ( var i=0; i<childs.length; i++ ) {
            if ( childs[i].namespaceURI == WOBZILLA_NS ) {
                if ( childs[i].localName == "sort") {
                    if ( sortConfigs == null)
                        sortConfigs = new Array();
                    sortConfigs.push(childs[i]);
                    continue;
                }
                if ( childs[i].localName == "with-param" ) {
                    if ( variableConfigs == null)
                        variableConfigs = new Array();
                    variableConfigs.push(childs[i]);
                    continue;
                }
            }
            if ( _type == FOR_EACH ) {
                if ( childConfigs == null )
                    childConfigs = new Array();
                childConfigs.push( childs[i] );
            }
        }

        if ( variableConfigs != null ) {
            var fact = _sm.lookup( "wobzilla.variableFactory" );
            _variableHandler = 
                fact.createVariableHandler(variableConfigs, _log, _context, _sm);
        }
        if (sortConfigs != null ) {
            var fact = _sm.lookup("wobzilla.sortFactory");
            _sortHandler = 
                fact.createSortHandler(sortConfigs, _log, _context, _sm);
        }
        if (childConfigs != null ) {
            _childConfig = WBZ_wrapChildConfigs(childConfigs);
            _factory = _sm.lookup( "wobzilla.componentFactory" );
        } else {
            _factory = _sm.lookup( "wobzilla.viewFactory" );
        }
    } 

    function equalsModel_Entry( model, entry ) {
        return model.isSameNode( entry.model );
    }

    this.init = function( outputNode, parentModel, insertAfter ) {
        _parentModel = parentModel;
        _outputNode = outputNode;
        if ( _variableHandler != null ) {
            _args = new DefaultContext();
            _variableHandler.init( _parentModel, _args );
        }
        if ( insertAfter != null ) {
            _markerStartNode = document.createTextNode("");
            WBZ_insertAfter( _outputNode, _markerStartNode, insertAfter );
            insertAfter = _markerStartNode;
        } 
        if ( _type == CALL_TEMPLATE ) {
            var entry = createEntry( parentModel
                                     , insertAfter
                                     , 1
                                     , 1
                                     );
            insertAfter = entry.outputEnd;
            _entries.push( entry );
        } else {
            var modelNodes = _resolver.getNodes( _parentModel, _xpath, _context );
            for ( var i=0; i < modelNodes.length; i++ ) {
                //log.debug("REPEATING: " + modelNodes[i].nodeName);
                var entry = createEntry( modelNodes[i]
                                         , insertAfter
                                         , i + 1
                                         , modelNodes.length 
                                         );
                insertAfter = entry.outputEnd;
                _entries.push( entry );
            }
        }

        _markerEndNode = document.createTextNode("");
        WBZ_insertAfter( _outputNode, _markerEndNode, insertAfter );
        if ( _sortHandler != null ) {
            _sortHandler.sort( _entries, relocate );
        } 
        initEntries();
        insertAfter = _markerEndNode;
        return insertAfter;
    }

    this.requery = function(changes) {
        var positionChanged = false;
        if ( _type != CALL_TEMPLATE ) {
            positionChanged = updateModel();
        }
        var variableChanged = false;
        // test if variables have changed
        if (_variableHandler != null) {
            // requery variables
            variableChanged = _variableHandler.requery(changes);
        }
        // create new changes event if neccessary
        if (variableChanged || changes.localVariableChanged == true 
            || (positionChanged && _type == FOR_EACH))
        {
            changes = WBZ_clone(changes);
            if ( _type != FOR_EACH) {
                // set new localVariable flag
                changes.localVariableChanged = false;
                // set new parameter flag
                changes.paramChanged = variableChanged;
            }
            changes.positionChanged = (positionChanged && _type == FOR_EACH);
        }
        for (var i = 0; i < _entries.length; i++) {
            if (_cutDisabled || WBZ_isTargeted( changes, _entries[i].model)) {
                _entries[i].view.requery( changes );
            }
        }
    }

    function updateModel() {
        var newModel = _resolver.getNodes( _parentModel, _xpath, _context );
        var modelChanged = !WBZ_equals( newModel, _entries, equalsModel_Entry );
        if ( modelChanged ) {
            var inserted = new Array();
            var removed = new Array();
            WBZ_diff( newModel, _entries, inserted, removed, equalsModel_Entry );
            for ( var i=0; i<removed.length; i++) {
                removeEntry( removed[i] );
                //log.info(i + ") removed: " + removed[i].model.getAttribute("name"));
                removed[i].model = null;
            }
            var newEntries = new Array();
            var oldEntrieIndex = 0;
            var insertIndex = 0;
            var lastOutput = _markerStartNode;
            for ( var i=0; i<newModel.length; i++) {
                var model = newModel[i];
                if ( insertIndex <= inserted.length 
                    && model.isSameNode( inserted[insertIndex] ))
                {
                    //log.info(i + ") Inserted: " + model.getAttribute("name"));
                    var newEntry = createEntry( newModel[i]
                                                , lastOutput
                                                , newEntries.length + 1
                                                , newModel.length
                                                );
                    newEntries.push( newEntry );
                    lastOutput = newEntry.outputEnd;
                    insertIndex ++;
                    continue;
                } else if (_entries.length > 0) {
                    while (true) {
                        var entry = _entries[oldEntrieIndex];
                        oldEntrieIndex++;
                        if (entry.model == null) //deleted
                            continue;
                        if (model.isSameNode( entry.model )) {
                            newEntries.push( entry );
                            lastOutput = entry.outputEnd;
                            break;
                        } 
                    }
                }
            }
            _entries = newEntries;
        }
        var positionChanged = false;
        if ( _sortHandler != null ) {
            positionChanged = _sortHandler.sort( _entries, relocate );
        } 
        if ( modelChanged ) {
            initEntries();
        }
        return positionChanged;
    }

    // validates the child views
    this.validate = function() {
        for (var i=0; i < _entries.length; i++) {
            _entries[i].view.validate();
        }
    }

    // validates the child views
    this.dispose = function() {
        for (var i=0; i < _entries.length; i++) {
            _factory.disposeComponent( _entries[i].view );
        }
    }
    
    function createEntry(model, insertAfter, position, last) {
        var childView;
        var entry = new Object();
        var positionContext = new PositionContext(1,1);
        var context = new DefaultContext( _context );
        context.put( "wobzilla.position", positionContext );
        
        if ( _type == FOR_EACH) {
            positionContext.position = position;
            positionContext.documentPosition = position;
            positionContext.last = last;
            childView = _factory.createComponent( _childConfig, _log, context, _sm);
        } else {
            // init parameters
            if (_args != null && _args.has("title"))
                _log.info( "Title: " + _args.get("title") );
            childView = _factory.createView(model
                                           ,_mode
                                           ,_log
                                           ,_context
                                           ,_sm
                                           ,_importsOnly != null
                                           ,_templateName
                                           ,_args
                                           );
        }
        var outputStart = document.createTextNode(""); 
        WBZ_insertAfter(_outputNode, outputStart, insertAfter);
        var outputEnd = document.createTextNode(""); 
        WBZ_insertAfter(_outputNode, outputEnd, outputStart);

        entry.view = childView; 
        entry.positionContext = positionContext;
        entry.model = model; 
        entry.outputStart = outputStart;
        entry.outputEnd = outputEnd;
        return entry;
    }

    /** relocate method for sorting*/
    function relocate( entry ) {
        //_log.info("relocate " + WBZ_serializeXML(entry.model))
        var following = entry.nextEntry;
        var lastNode =  (following != null) ? following.outputStart : _markerEndNode;
        var outputStart = entry.outputStart;        
        var outputEnd = entry.outputEnd;        
        if (outputEnd == null) {
            outputEnd = outputStart;
        }
        var nodes = new Array();
        var current = outputStart;
        while (true) {
            var next = current.nextSibling;
            nodes.push(current);
            _outputNode.removeChild(current);
            if (current.isSameNode(outputEnd))
                break;
            current = next;
        }
        for (i=0;i<nodes.length;i++) {
            WBZ_insertBefore( _outputNode, nodes[i], lastNode);
        }
    }

    function initEntries() {
        for (var i = 0; i < _entries.length; i++) {
            var entry = _entries[i];
            if ( entry.initialized  == null) {
                entry.view.init( _outputNode, entry.model, entry.outputStart );
                entry.initialized = true;
            }
        }
    }

    function removeEntry(entry) {
        _factory.disposeComponent( entry.view );
        var outputStart = entry.outputStart;        
        var outputEnd = entry.outputEnd;        
        var current = outputStart;
        while (true) {
            var next = current.nextSibling;
            _outputNode.removeChild(current);
            if (current.isSameNode(outputEnd))
                break;
            current = next;
        }
    }
}
