function WbzChoose() {
    var _log;
    var _sm;
    var _cf;
    var _resolver;
    var _context;

    var _childView;
    var _parentModel;
    var _outputNode;
    var _markerStartNode;
    var _markerEndNode;

    var _conditions;

    var _viewConfigs;
    var _viewConfig;

    this.enableLogging = function(logger) {
        _log = logger;
    }

    this.contextualize = function(context) {
        _context = context;
    }

    this.service = function(serviceManager) {
        _sm = serviceManager;
        _resolver = _sm.lookup("wobzilla.xpath");
        _cf = _sm.lookup("wobzilla.componentFactory");
    }

    this.configure = function(config) {
        // we have an if attribute
        if (config.localName == "if") {
            _conditions = new Array(1);
            _viewConfigs = new Array(1);
            _conditions[0] = config.getAttribute("test");
            _viewConfigs[0] = WBZ_wrapConfig(config);
        } else {
            _conditions = new Array();
            _viewConfigs = new Array();
            var childs = config.childNodes;
            for (var i=0;i<childs.length;i++) {
                if (childs[i].namespaceURI != WOBZILLA_NS) {
                    continue;
                }
                if (childs[i].localName == "when") {
                    _conditions.push(childs[i].getAttribute("test"));
                    _viewConfigs.push(WBZ_wrapConfig(childs[i]));
                } else if (childs[i].localName == "otherwise"){
                    _conditions.push(null);
                    _viewConfigs.push(WBZ_wrapConfig(childs[i]));
                }
            }
        }
    }

    this.init = function(outputNode, parentModel, insertAfter) {
        _parentModel = parentModel;
        _outputNode = outputNode;
        if ( insertAfter != null ) {
            _markerStartNode = document.createTextNode("");
            WBZ_insertAfter( _outputNode, _markerStartNode, insertAfter );
            insertAfter = _markerStartNode;
        } 
        _markerEndNode = document.createTextNode("");
        WBZ_insertAfter( _outputNode, _markerEndNode, insertAfter );
        _viewConfig = findViewConfig();
        myinit(insertAfter);
        return _markerEndNode;
    }

    this.requery = function( changes ) {
        var newConfig = findViewConfig();
        if ( newConfig != _viewConfig) {
            _viewConfig = newConfig;
            rebuild();
            return;
        }
        if ( _childView!= null && _childView.requery) {
            _childView.requery( changes );
        }
    }

    // validates the child views
    this.validate = function() {
        if ( _childView == null) {
            return;
        }
        _childView.validate();
    }

    this.dispose = function() {
        _cf.disposeComponent( _childView );
    }

    function findViewConfig() {
        for (var i = 0 ; i < _conditions.length; i++ ) {
            // allways return if wbz:otherwise
            if ( _conditions[i] == null )
                return _viewConfigs[i];
            // return if test condition holds
            if ( _resolver.getBoolean( _parentModel, _conditions[i] , _context ) )
                return _viewConfigs[i];
        }
        // no condition holds
        return null;
    }

    function myinit( insertAfter ) {
        if ( _viewConfig == null ) {
            _childView == null;
            return insertAfter;
        }

        _childView = _cf.createComponent( _viewConfig, _log, _context, _sm );
        insertAfter = _childView.init( _outputNode, _parentModel, insertAfter);
        return insertAfter;
    }
    
    function rebuild() {
        if ( _log.debugging )
            _log.debug("Rebuilding Choose");
        if ( _childView != null )
            _cf.disposeComponent( _childView );

        var currentNode = ( _markerStartNode != null ) ? 
            _markerStartNode.nextSibling : _outputNode.firstChild;

        while ( currentNode != null 
                && !currentNode.isSameNode( _markerEndNode ) ) 
        {
            var removeNode = currentNode;
            currentNode = currentNode.nextSibling; 
            _outputNode.removeChild(removeNode);
        }
        myinit( _markerStartNode );
    }
}

