/** 
<code>WbzMapper</code> is the root-element of a
mapping-configuration. It is usally instanciated from the
WobzillaEngine or another <code>WbzMapper</code> (during an import).
The configuration of <code>WbzMapper</code> is a complete mapping-description,
but it only manges the direct sub-elements, such as import instructions, global
variables and matching rules. 

    <h4>Attributes:</h4>
    <table width="90%">
      <thead>
          <tr>
            <td>attribute</td>
            <td>type</td>
            <td>description</td>
          </tr>
      </thead>
      <tbody>
          <tr>
            <td><strong>version</strong></td>
            <td><code>QName</code></td>
            <td>The wobzilla-version, that is requiered for a proper execution. 
            Also permitted are intervals like 0.1-0.4 or 1.0+
            </td>
          </tr>
      </tbody>
    </table>

    <h4>Child-Elements:</h4>
    <table width="90%">
      <thead>
          <tr>
            <td>configuration</td>
            <td>count</td>
            <td>description</td>
          </tr>
      </thead>
      <tbody>
          <tr>
            <td><strong>import</strong></td>
            <td>0-*</td>
            <td>The imported mapping-configuration. A new
            <code>WbzMapper</code> instance is created for each
            import-instruction. The path to the mapping file is passed
            in the <code>hfref</code>-attribute of the import element.</td>
          </tr>
          <tr>
            <td><strong>variable/param</strong></td>
            <td>0-*</td>
            <td>The definition of global variables, see <a href="wobzilla_variableFactory.html">wobzilla_variableFactory</a> for more information about variables. When placed
            as direct childs of <code>WbzMapper</code> variables can be used for
            <a href="validation.html">validation of global constraints.</a></td>
          </tr>
          <tr>
            <td><strong>template</strong></td>
            <td>1-*</td>
            <td>The matching rules. Each matching rule contains either a <code>match</code> or a <code>name</code> attribute and an arbirary view-configuration as a child-node. See
            <a href="wobzilla_viewFactory.html">wobzilla_viewFactory</a> for further information.</td></td>
          </tr>
      </tbody>
    </table>


@requires wobzilla_componentFactory
@requires wobzilla_xmlloader 

@provides wobzilla_viewFactory
@provides wobzilla_variableFactory
@provides wobzilla_sortFactory
@provides wobzilla_xpath

@constructor
*/
function WbzMapper() {
   function dummy();
}


/**
   This implementation of the <code>wobzilla.xpath</code> interface uses
   Mozillas DOM3-xpath-api for evaluating the xpath-expressions.
   Unfortunatly this way Mozilla doesn't provide a way of using variables
   in xpath expressions. Use <a href="ContextXPathResolver.html">ContextXPathResolver</a>
   as a wrapper for this resolver, to use variables.
   
  @implements wobzilla_xpath
  @constructor
*/
function DefaultXPathResolver(document,root) {
   function dummy();
}

/**
   This implementation replaces all variable-values in <em>xpath</em>-
   expressions with a correspondig entries in the contex before delegating
   the call to the parent-resolver. 
   <p>Variable-values are taken from
   the context-entries <code>wobzilla.local</code>, <code>wobzilla.global</code>
   and <code>wobzilla.arguments</code>.
   It also replaces all occurrences of <code>position()</code> and
   <code>last()</code> with the values provided by the context entry
   in <code>wobzilla.position</code>.
   </p>

   <p>Mozilla does not provide a way to specify variables or position functions 
   in <em>xpath</em>-expression (at least not through the exposed apis). This class
   works around this by replacing the variable-names with the variable-values before
   further processing the xpath expression.
   </p>
   <p>

   <strong>Note:</strong> This method currently replaces all
   occurences of $ followed by a word, even if the expression is
   quoted e.g. '$test'. To use $ in the output you have to use the
   concat function. Write concat('$','test') instead of '$test'. 
  </p>

  @implements wobzilla_xpath
  @see wobzilla_variableHandler
  @constructor
*/
function ContextXPathResolver(parent,global) {
    function dummy();
}

/**  This services is responsilbe for the creation of new <em>views</em> that are
     selected with the help of template-rules. The <em>view</em>-configuration 
     can be found with the <code>findRule</code> method. 
     For a detailed description of the selection algorithm see the
     <a href="http://www.w3.org/TR/xslt">XSLT-specification</a>.
     @constructor
 */
function wobzilla_viewFactory(INTERFACE_ONLY_NO_CONSTRUCTOR) {
    /** selects a template rule from the given contextNode or templateName (if set).
       First checks all template-rules of the current WbzMapper-Object. 
       If no rule is found, the imported stylesheets will be searched.
      @param contextNode the contextNode used for matching
      @param mode the mode of the template rule
      @param importedRulesOnly only search rules of imported stylesheets
      @param templateName selects a rule by name without matching the contextNode.
     */
    this.findRule = function(contextNode,mode,importedRulesOnly,templateName);
    /** finds a rule for the contextNode and creates the appropiate
        view with the help of the <code>componentFactory</code>. The
        contents of the Rule will be used as a configuration node for
        the componentFactory. If there is more than one content-node
        (or an element with a namespace different from
        "http://wobzilla.sf.net/views" a configuration for a wrapping
        WbzContainer will be created.
     * @param contextNode will be passed to findRule
     * @param mode will be passed to findRule
     * @param logger will be passed to the <code>createComponent</code> method
     * @param context will be passed to the <code>createComponent</code> method
     * @param serviceManager will be passed to the <code>createComponent</code> method
     * @param importedRulesOnly will be passed to findRule
     * @param templateName will be passed to findRule
     * @param args
     * @see componentFactory
     */
    this.createView = function(contextNode,mode,logger,context,serviceManager,importedRulesOnly,templateName, args);
}

/**  A Variable-Handler can automate variable-handling in container-elements.
     It stores the variable values into an context-object.
     Assume the following Configuration:
<pre>
  &lt;variable name="books">
   &lt;for-each select="author/books">
    &lt;value-of select="&#64;name">;
   &lt;/for-each>
  &lt;/variable>
  &lt;variable name="count" select="count(author/books)"/>
  &lt;variable name="book-writer" select="$count > 0"/>
</pre>
The context will hold the following values after initialization of the variables:
<ul>
 <li><code>outputContext.get("books")</code> returns "'Book A;Book B;'".</li>
 <li><code>outputContext.get("count")</code> returns "2".</li>
 <li><code>outputContext.get("book-writer")</code> returns "true()".</li>
</ul>
  @constructor
 */
function wobzilla_variableHandler(INTERFACE_ONLY_NO_CONSTRUCTOR) {
    /** initialize Variables and writes the values into the
     <code>ouputContext</code>. 

    */
    this.init = function(modelNode,outputContext);

    /** updates variable-contents and writes the updated values into the ouputContext.
        @return true if any variable has changed.  */
    this.requery = function(changes);

    /** updates all variables that are configured with 
        <code>evaluate="onvalidate"</code>. After that it converts the contents of all 
        validate-variables (variables with <code>validation="true"</code>) to boolean and
        test if the values are all <code>true</true>.
        @return true if validation was successfull for all validate-variable.  */
    this.validate = function(changes);
}

/**  Factory for VariableHandler-Objects. Creates an VariableHandler
     for an Array of variableConfiguration nodes.  Example
     Configuration: 
<pre> 
 &lt;wbz:variable name="lastname" select="author/lastname"/>
 &lt;wbz:variable name="fistname" select="author/firstname"/>
</pre>

or 

<pre> 
 &lt;param name="id" select="&#64;id"/>
</pre>

     @see wobzilla_variableHandler
     @constructor
 */
function wobzilla_variableFactory(INTERFACE_ONLY_NO_CONSTRUCTOR) {
    /** creates a new new <code>wobzilla.variableHandler</code> for 
      * given variable-configurations and a context-object.
      *  @return wobzilla_variableHandler */
    this.createVariableHandler = function(variableConfigs,log,context,sm);
}

/**  Factory for <code>sort_handler</code> objects.
 It can be configured with an array of configuration nodes.
 For more information about sorting see the XSLT-specification.
 Example with two configuration nodes:
 <pre>
  &lt;wbz:sort select="last-name" lang="de">
  &lt;wbz:sort select="id" order="ascending" data-type="numeric">
 </pre>
  @see wobzilla_sortHandler
  @constructor
 */
function wobzilla_sortFactory(INTERFACE_ONLY_NO_CONSTRUCTOR) {
/** creates a new new <code>wobzilla.sortHandler</code> for 
 *    a given configuration-array and initialize it with the
 * specified log, context and serviceManager.
 * @return wobzilla_sortHandler
*/
    this.createSortHandler = function(sortConfigs,log,context,serviceManager);
}

/** The sort function manipulates the following properties of the
 entry objects of a given list:
 <ul>
  <li><code>sortKeys</code>: Used internaly by the sortHandler for
     storing the sort keys. A resort is necessary if the one of the sort
     keys has changed.
  </li>
  <li><code>nextEntry</code>: pointer to the next entry</li>
  <li><code>positionContext</code>: the positionContext entry</li>
 </ul>
  @see PositionContext
  @constructor
 */
function wobzilla_sortHandler(INTERFACE_ONLY_NO_CONSTRUCTOR) {
    /** Sorts the entries. First it recalculates the sort-keys of
        every entry in the passed list and adjusts the
        <code>nextEntry</code> property for all entries. 
        This method doesn't rearrange the array. It calls the
        <code>relocateMethod</code> if the <code>nextEntry</code>
        property has changed and passed the corresponding entry.

       @return <code>true</code> if the position have changed and <code>false</code> otherwise
     */
    this.sort = function(entries, relocateMethod);
}


/** This service provides an interface for evaluating <em>xpath</em>-expressions
    on the data-model.
  @constructor
 */
function wobzilla_xpath(INTERFACE_ONLY_NO_CONSTRUCTOR) {
    /** evaluates the xpath expression on the <code>modelNode</code> and returns
        the selected nodes as an array. The <code>context</code> can provide additional 
        information, like local and global variables. 
     */
    this.getNodes = function(modelNode, xpath, context);
    /** evaluates the xpath expression on the <code>modelNode</code> and returns
        the first selected node. The <code>context</code> can provide additional 
        information, like local and global variables. Returns <code>null</code>
        if no node was found.
     */
    this.getNode = function(modelNode, xpath, context);
    /** same as <code>getNode</code> but throws an exception when
        no node was found.*/
    this.getNodeSafe = function(modelNode, xpath, context);
    /** evaluates the xpath expression on the <code>modelNode</code>
        and returns the first selected node converted to a number. The
        <code>context</code> can provide additional information, like
        local and global variables.
     */
    this.getNumber = function(modelNode, xpath, context);
    /** evaluates the xpath expression on the <code>modelNode</code>
        and returns the first selected node converted to a string. The
        <code>context</code> can provide additional information, like
        local and global variables.
     */
    this.getText = function(modelNode, xpath, context);
    /** evaluates the xpath expression on the <code>modelNode</code>
        and returns the first selected node converted to a boolean
        value. The <code>context</code> can provide additional
        information, like local and global variables.
     */
    this.getBoolean = function(modelNode, xpath, context);
}

