function WbzTextField() {
    var _log;
    var _sm;
    var _context;

    var _parentModel;
    var _model;
    var _xpath;
    var _outputNode;
    var _cols;
    var _rows;
    var _maxlength;
    var _xmlContent = false;
    var _changing; // Indicates if the TextField is commiting updates
    var _resolver;

    this.enableLogging = function(logger) {
        _log = logger;
    }

    this.contextualize = function(context) {
        _context = context;
    }

    //public functions
    this.configure = function(config) {
        _xpath = config.getAttribute("select");
        _cols = config.getAttribute("cols");
        _maxlength = config.getAttribute("maxlength");
        if ( _cols == null  || _cols == "" ) {
            _cols = _maxlength;
        }
        if ( _cols == null ) {
            _cols = 40;
        }
        _rows = config.getAttribute("rows");
        if ( _rows == null || _rows == "" ) {
            _rows = 1;
        }
        if (config.getAttribute("xml") == "true") {
            _xmlContent = true;
        }
    }

    this.service = function(serviceManager) {
        _sm = serviceManager;
        _resolver = _sm.lookup("wobzilla.xpath");
    }
    
    this.init = function(outputNode,parentModel,insertAfter) {
        _parentModel = parentModel;
        
        if ( _rows > 1 ) {
          _outputNode = document.createElementNS(HTML_NS,"textarea");
          _outputNode.setAttribute("rows", _rows);
          _outputNode.setAttribute("cols", _cols);
          _outputNode.setAttribute("name", "test");
        } else {
          _outputNode = document.createElementNS(HTML_NS,"input");
          _outputNode.setAttribute("size", _cols);
          if ( _maxlength > 0 ) {
              _outputNode.setAttribute("maxlength", _maxlength);
          }
        }
        //outputNode = document.createElementNS(XUL_NS,"textbox");
        _outputNode.setAttribute("class","input");
        _outputNode.addEventListener("blur",onblur,true);
        _outputNode.addEventListener("keypress",onkeypress,true);

        WBZ_insertAfter(outputNode,_outputNode,insertAfter);
        insertAfter = _outputNode;

        _model = _resolver.getNode(_parentModel, _xpath, _context);
        initContent();
        
        return insertAfter;
    }

    function initContent() {
        if ( _model == null ) {
            setValue("");
            return;
        }
        var childs = _model.childNodes;
        for (var i=0;i<childs.length;i++) {
            if ( childs[i].nodeType == ELEMENT_NODE )
                _xmlContent = true;
        }
        if ( _xmlContent ) {
            setValue( getFromXML() );
        } else if ( _model.nodeType == ELEMENT_NODE ) {
            var value = "";
            var childs = _model.childNodes;
            for (var i=0;i<childs.length;i++) {
                if (childs[i].nodeType == TEXT_NODE)
                    value += childs[i].nodeValue;
            }
            setValue(value);
        } else if ( _model.nodeType == ATTRIBUTE_NODE ) {
            setValue( _model.nodeValue );
        } else {
            throw new Error("NodeType not supported");
        }
    }

    this.requery = function(changes) {
        if ( _changing )
           return;
        var newModel = _resolver.getNode(_parentModel, _xpath, _context);
        // check if new model is available
        if ( _model != newModel || _model == null ) {
            _model = newModel;
            initContent();
            return;
        } 
        var value = null;
        for (var i=0;i<changes.events.length;i++) {
            var event = changes.events[i];
            
            if ( _model.nodeType == ATTRIBUTE_NODE ) {
                if (event.target.isSameNode( _model.ownerElement )
                    && event.type == "DOMAttrModified"
                    && event.attrName == _model.nodeName
                    ) 
                {
                    value = event.newValue;
                }
            } else {
                if (!WBZ_isAncestorOf( _model, event.target)) {
                    continue;
                } 
                if ( _xmlContent ) {
                    value = getFromXML();
                } else {
                    var newValue = "";
                    var childs = _model.childNodes;
                    for (var i=0;i<childs.length;i++) {
                        if (childs[i].nodeType == TEXT_NODE) {
                            newValue += childs[i].nodeValue;
                        }
                    }
                    value = newValue;
                }
            }
        }
        if (value != null) {
            setValue(value);
        }
    }
        
    this.validate = function() {
        myvalidate();
    }

    this.dispose = function() {
    }

    function myvalidate() {
        _changing = true;
        try {
            if ( _log.debugging )
                _log.debug("Validate " + getValue());
            
            if ( _model == null || _model.nodeType == ATTRIBUTE_NODE) {
                validateAttribute();
            } else if ( _xmlContent ) {
                validateXML();
            } else if ( _model.nodeType == ELEMENT_NODE) {
                validateElement();
            }
        } finally {
            _changing = false;
        }
    }

    function validateXML() {
        var originalValue = getFromXML();
        if (originalValue != getValue()) {
            var newValue;
            try {
                newValue = parseToXML(getValue());
            } catch (ex) {
                if ( window.confirm( "Invalid XML-content."
                                     + "Do you want to return to "
                                     + "the last valid content?\n " + getValue()) ) 
                {
                    setValue( getFromXML() );
                }
                return;
            } 
            _sm.lookup("wobzilla.transaction").start();
            WBZ_removeAll( _model );
            _model.appendChild(newValue);
            _sm.lookup("wobzilla.transaction").commit();
        }
    }

    function validateElement() {
        var value = "";
        var childs = _model.childNodes;
        for ( var i = 0; i < childs.length; i++ ) {
            if ( childs[i].nodeType == TEXT_NODE )
                value += childs[i].nodeValue;
        }
        var currentValue = getValue();
        
        if ( _rows == 1 ) {
            value = removeBreaks(value);
            currentValue = removeBreaks(currentValue);
        }
        if ( value != currentValue ) {
            _sm.lookup("wobzilla.transaction").start();
            WBZ_removeAll( _model );
            //debug(getValue());
            _model.appendChild(document.createTextNode(getValue()));
            _sm.lookup("wobzilla.transaction").commit();
        }
    }

    function validateAttribute() {
        // Try to guess new Attribute from XPath Expression
        if ( _model == null ) {
            if ( _xpath.charAt(0) != '@' ) {
                throw new Error("Can only create optional attribute nodes.");
            }
            // Don't create Attribute if textfield is empty
            if (getValue() == "")
                return;

            try {
                var attrName = _xpath.substr(1);
                _sm.lookup("wobzilla.transaction").start();
                _parentModel.setAttribute(attrName,getValue());
                _model = _parentModel.getAttributeNode(attrName);
                _sm.lookup("wobzilla.transaction").commit();
            } catch (ex) {
                throw new Error("Can't create attribute for xpath " + _xpath + ex.message);
            }
        } else if ( _model.nodeValue != getValue() ) {
            var attributeName = _model.localName;
            var ns = _model.namespaceURI;
            _sm.lookup("wobzilla.transaction").start();
            // The following doesn't work due some kind of bug:
            //_model.value = getValue();
            // so we must use this hack
            if (ns != null) {
                if ( _model.prefix != null ) {
                    attributeName = _model.prefix + ":" + attributeName;
                }
                //_log.debug(attributeName + ns);
                _model.ownerElement.setAttributeNS(ns,attributeName,getValue());
            } else {
                _model.ownerElement.setAttribute(attributeName,getValue());
            }
            _sm.lookup("wobzilla.transaction").commit();
            //_log.info("Attribute CHANGED");
        } 
    }

    function removeBreaks(text) {
        if ( _model.nodeType == ATTRIBUTE_NODE )
            return text;
        // remove linebreaks and tabs
        text = text.replace(/\n/g,'');
        text = text.replace(/\t/g,'');
        // normalize space
        text = text.replace(/^\s*/,'');
        text = text.replace(/\s*$/,'');
        return text;
    }

    function setValue(value) {
        if (_outputNode.value != value) {
            if ( _rows == 1 ) {
                value = removeBreaks(value);
            }
            _outputNode.value = value;
        }
    }

    function getValue() {
        return _outputNode.value;
    }

    function onblur() {
        try {
            myvalidate();
        } catch (ex) {
            _log.error(ex);
        }
    }

    function onkeypress( event ) {
        if (event.keyCode == event.DOM_VK_RETURN)
            myvalidate();
    }

    function getFromXML() {
        var content = WBZ_getFirstElement( _model );
        return _sm.lookup("wobzilla.serializer").serializeToString(content);
    }

    function parseToXML( string ) {
        var result = _sm.lookup("wobzilla.parser").parseString( string, _model.ownerDocument);
        return result;
    }


}

