/**
 Provides the default implementation of the avalon-interfaces
 @avalon.logger
 @avalon.serviceManager
 @avalon.context
 */

DEBUG_LEVEL=1;
INFO_LEVEL=2;
WARNING_LEVEL=3;
ERROR_LEVEL=4;
var VERBOSE_LEVEL = DEBUG_LEVEL;
var DEBUGGING = (VERBOSE_LEVEL <= DEBUG_LEVEL);

var HTML_NS="http://www.w3.org/1999/xhtml";
/** 
   DefaultLogger Object. Appends logging message to a specified 
   HTML-Node.
  @implements avalon_logger
  @constructor 
 */
function DefaultLogger(category,outputBody) {
    var category;
    var outputNode = outputBody;
    this.debug = function(message) {
        if (this.debugging) {
            if (message == null) {
                message = "null";
            }
            var node = document.createElementNS(HTML_NS,"span");
            node.setAttribute("style","color: #AAAAAA;");
            node.appendChild(createTimePrefix("DEBUG",category));
            node.appendChild(convertNode(message));
            logPrintNode(node);
        }
    }
    
    this.info = function(message) {
        if (VERBOSE_LEVEL > INFO_LEVEL) {
            return;
        }
        if (message == null) {
            message = "null";
        }
        var node = document.createElementNS(HTML_NS,"span");
        node.setAttribute("style","color: #5553c6;");
        node.appendChild(createTimePrefix("INFO",category));
        node.appendChild(convertNode(message));
        logPrintNode(node);
    }

    this.warn = function(message) {
        if (VERBOSE_LEVEL > INFO_LEVEL) {
            return;
        }
        if (message == null) {
            message = "null";
        }
        var node = document.createElementNS(HTML_NS,"span");
        node.setAttribute("style","color: #e89600;");
        node.appendChild(createTimePrefix("WARNING",category));
        node.appendChild(convertNode(message));
        logPrintNode(node);
    }

    this.error = function(ex,text) {
        try {
            var node = document.createElementNS(HTML_NS,"div");
            node.setAttribute("style","background-color:#FFAAAA;");
            node.appendChild(createTimePrefix("ERROR",category));
            if (text != null) {
                node.appendChild(document.createTextNode(text + " "));
            }
            if (ex != null) {
                if (ex.name != null && ex.message != null) {
                    node.appendChild(document.createTextNode(ex.name + " " +ex.message));
                } else {
                    node.appendChild(document.createTextNode(ex));
                }
                if (ex.line != null)
                    node.appendChild(document.createTextNode("line: " + ex.line));
                var stackMessage;
                if (ex.stack != null && ex.stack.length>0) {
                    stackMessage = ex.stack;
                } else {
                    try {
                        throw new Error();
                    } catch (stackex) {
                        stackMessage  = stackex.stack;
                    }
                }
                var stack = document.createElementNS(HTML_NS,"div");
                stack.setAttribute("style","margin-left:15pt;");
                stack.appendChild(convertNode(stackMessage));
                node.appendChild(stack);
            } else {
                node.appendChild(document.createTextNode("blad "));
            }
            logPrintNode(node);
        } catch (exception) {
            alert("Can't log error " + ex + "\n" + exception);
        }
    }

    this.getChildLogger = function(childCategory) {
        return new DefaultLogger(category + "." + childCategory,outputNode);
    }

    function logSimpleError(message,file, line) {
        logError(message + "\n___" + file + " Line: " + line)
    }

    function getTimeString() {
       var now = new Date();
       var minutes = now.getMinutes();
       var seconds = now.getSeconds();
       var milliseconds = now.getMilliseconds();
       var result = "";
       if (minutes <10)
           result +="0";
       result += minutes + ":";
       if (seconds <10)
           result +="0";
       result += seconds + ":";
       if (milliseconds <10)
           result +="0";
       if (milliseconds <100)
           result +="0";
       if (milliseconds <1000)
           result +="0";
       result += milliseconds;
       return result;
    }

    function createTimePrefix(levelName,category) {
        var prefix = levelName + " " + getTimeString();
        if (category != null)
            prefix = prefix + " [" + category + "]";
        prefix = prefix + "  "
        return document.createTextNode(prefix);
    }

    function convertNode(message) {
        var node = document.createElementNS(HTML_NS,"span");
        var i =0;
        while (i<message.length) {
            var endPos = message.indexOf("\n",i);
            if (endPos < 0) {
                endPos = message.length;
            }
            node.appendChild(document.createTextNode(message.substring(i,endPos)));
            i = endPos + 1;
            if (i< message.length)
                node.appendChild(document.createElementNS(HTML_NS,"br"));
        }
        return node;
    }


    function logPrintNode(node) {
        try {
            element = document.createElementNS(HTML_NS,"p");
            outputNode.insertBefore(document.createElementNS(HTML_NS,"br")
                                    ,outputNode.firstChild);
            outputNode.insertBefore(node,outputNode.firstChild);
        } catch (e) {
            alert("Can't print into log window " + e);
        }
    }
}

DefaultLogger.prototype.__defineGetter__(
                                        "debugging"
                                        ,function() {return DEBUGGING;}       
                                        );



/** 
  @implements avalon_serviceManager
  @constructor
 */
function DefaultServiceManager( _parentManager ) {
    var _services = new Array();

    this.lookup = function(role) {
        var service = _services[role];
        if (service != null) {
           return service;
        }
        if ( _parentManager != null ) {
           return _parentManager.lookup(role);
        }
        throw new Error("Service for role " + role + " not found!");
    }

    this.hasService = function(role) {
        if ( _services[role] != null) {
            return true;
        }
        if ( _parentManager != null ) {
           return _parentManager.hasService(role);
        } 
        return false;
    }

    this.put = function(role,service) {
        _services[role] = service;
    }
}

/**
  DefaultContext 
  @implements avalon_context
  @constructor
 */
function DefaultContext( _parentContext ) {
    var _context = new Array();

    this.get = function(key) {
        var entry = _context[key];
        if (entry != null) {
           return entry;
        }
        if ( _parentContext != null ) {
           return _parentContext.get(key);
        }
        throw new Error("Context-entry for " + key + " not found!");
    }

    this.has = function(key) {
        if ( _context[key] != null) {
            return true;
        }
        if ( _parentContext != null ) {
            return _parentContext.has(key);
        }
        return false;
    }

    this.put = function(key,entry  ) {
        _context[key] = entry;
    }
}



