<?xml version="1.0"?>
<!--  Stylesheet generates wobzilla-template from XML-Schema.
It generates one matching rule for each simpleType and complexType. 
It currently respects the doc:label annotation, see entityMapping.xsl.
You can pass a language param to choose from the labels: lang=de
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:wbz="http://wobzilla.sf.net/views"
    xmlns:html="http://www.w3.org/1999/xhtml"
    xmlns:doc="http://wobzilla.sf.net/schema-annotation"
    exclude-result-prefixes="doc xsd"
    version="1.0">
 <xsl:import href="entityMapping.xsl"/>    
 <xsl:output method="xml" indent="yes"/>
 <xsl:output name="entity-map" method="text" indent="no"/>	

 <!-- Create entity mapping and rules for all complexTypes and simpleTypes -->
 <xsl:template match="/xsd:schema">
   <xsl:call-template name="doctype"/>
   <wbz:stylesheet> 
    <xsl:apply-templates select="//xsd:complexType"/>
    <xsl:apply-templates select="//xsd:simpleType"/>
    <!-- default rule matches document root -->
    <wbz:template match="/"> 
      <wbz:apply-templates/>
    </wbz:template>
   </wbz:stylesheet> 
 </xsl:template>

 <!-- returns all elements that use the attribute.
     Elements are separated with a '|'.
     Example return value: (bookmark|folder) -->
 <xsl:template name="findParentElements">
   <xsl:param name="node" select="./parent::node()"/>
   <xsl:choose>
    <!-- parent is an element -->
    <xsl:when test="$node[local-name()='element']">
     <xsl:value-of select="concat($node/@name,$node/@ref)"/>
    </xsl:when>
    <!-- parent is a complextype -->
    <xsl:when test="$node[local-name()='complexType' and @name]">
      <xsl:variable name="name" select="$node/@name"/>    
      <xsl:value-of select="'('"/>
      <xsl:for-each select="//xsd:element[@type=$name]">
        <xsl:if test="position()&gt;1">
          <xsl:value-of select="'|'"/>
        </xsl:if>
        <xsl:value-of select="@name"/>
      </xsl:for-each>
      <xsl:value-of select="')'"/>
    </xsl:when>
    <!-- process parent of parent -->
    <xsl:otherwise>
      <xsl:call-template name="findParentElements">
        <xsl:with-param name="node" select="$node/parent::node()"/>
      </xsl:call-template>
    </xsl:otherwise>
   </xsl:choose>
 </xsl:template>

 <!-- Create one rule for each simpleType -->
 <xsl:template match="xsd:simpleType">
  <xsl:variable name="name" select="@name"/>
  <!-- select all attributes or elements that refer to that type -->
  <xsl:variable name="matched">
      <xsl:for-each select="//xsd:attribute[@type=$name]|//xsd:element[@type=$name]">
        <xsl:if test="position()&gt;1">
          <xsl:value-of select="'|'"/>
        </xsl:if>
        <xsl:if test="local-name()='attribute'">
          <xsl:call-template name="findParentElements"/>
          <xsl:value-of select="'/@'"/>
        </xsl:if>
        <xsl:value-of select="@name"/>
      </xsl:for-each>
  </xsl:variable> 

  <!-- Create rule for simpleType-->
  <wbz:template match="{$matched}"> 
    <wbz:view class="WbzTextField" select=".">
      <xsl:choose>
        <xsl:when test="@doc:rows">
        </xsl:when>
        <xsl:when test="@doc:length">
          <xsl:attribute name="cols"><xsl:value-of select="@doc:length"/></xsl:attribute>
        </xsl:when>
      </xsl:choose>
    </wbz:view>
  </wbz:template>
 </xsl:template>

 <!-- Create one rule for each complexType -->
 <xsl:template match="xsd:complexType">
   <xsl:variable name="matched"> 
    <xsl:choose>
     <xsl:when test="@name">
      <!-- select all elements that refer to that type -->
      <xsl:variable name="name" select="@name"/>
      <xsl:for-each select="//xsd:element[@type=$name]">
        <xsl:if test="position()&gt;1">
          <xsl:value-of select="'|'"/>
        </xsl:if>
        <xsl:value-of select="@name"/>
      </xsl:for-each>
     </xsl:when>
     <xsl:otherwise>
       <!-- anonymous types use their parent element name-->
       <xsl:value-of select="parent::node()/@name"/>
     </xsl:otherwise>
    </xsl:choose>
   </xsl:variable> 

   <!-- Create rule for complexType-->
   <wbz:template match="{$matched}"> 
      <wbz:container>
<!-- Table layout for attributes -->
        <html:table border="0">
         <xsl:for-each select="xsd:attribute">
          <xsl:variable name="type" select="@type"/>
          <html:tr>
           <html:td><xsl:call-template name="entity-label"/>:</html:td>
           <html:td><wbz:apply-templates select="@{@name}"/></html:td>
          </html:tr>
         </xsl:for-each>
        </html:table>
<!-- Div layout for attributes
         <xsl:for-each select="xsd:attribute">
          <xsl:variable name="type" select="@type"/>
          <html:div>
           <xsl:call-template name="entity-label"/>:
           <wbz:apply-templates select="@{@name}"/>
          </html:div>
         </xsl:for-each>
-->

        <xsl:apply-templates select="xsd:choice|xsd:sequence"/>
      </wbz:container>
   </wbz:template>
 </xsl:template>

 <!-- Default component for string-types 
e.g. <attribute type="xsd:string"/> -->
 <xsl:template match="*[@type='xsd:string']" mode="baseType">
   <wbz:view class="WbzTextField" select="."/>
 </xsl:template>

 <!-- create xpath expressions that matches all previous siblings -->
 <xsl:template name="insertPoint">
  <xsl:variable name="insertAt"> 
   <xsl:if test="parent::xsd:sequence">
    <xsl:for-each select="preceding-sibling::xsd:element|preceding-sibling::xsd:choice">
     <xsl:if test="local-name()='element'">
       <xsl:value-of select="concat('|',@ref,@name)"/>
     </xsl:if>
     <xsl:if test="local-name()='choice'">
      <xsl:for-each select="xsd:element">
        <xsl:value-of select="concat('|',@ref,@name)"/>
      </xsl:for-each>
     </xsl:if>
    </xsl:for-each>
   </xsl:if>
   <xsl:if test="parent::xsd:choice">
    <xsl:for-each select="parent::node()/(preceding-sibling::xsd:element|preceding-sibling::xsd:choice)">
     <xsl:if test="local-name()='element'">
       <xsl:value-of select="concat('|',@ref,@name)"/>
     </xsl:if>
     <xsl:if test="local-name()='choice'">
      <xsl:for-each select="xsd:element">
        <xsl:value-of select="concat('|',@ref,@name)"/>
      </xsl:for-each>
     </xsl:if>
    </xsl:for-each>
   </xsl:if>
  </xsl:variable> 
  <xsl:value-of select="substring-after($insertAt,'|')"/>
 </xsl:template>

 <!-- handle element maxOccurs -->
 <xsl:template match="xsd:element">
  <xsl:variable name="matched"> 
   <xsl:value-of select="concat(@ref,@name)"/>
  </xsl:variable> 
  <xsl:variable name="insertAt"> 
    <xsl:call-template name="insertPoint"/>
  </xsl:variable> 
  <xsl:choose>
   <xsl:when test="@maxOccurs&gt;1 or @maxOccurs='unbounded'">
    <wbz:view class="WbzRepeat" select="{$matched}" insertAt="{$insertAt}">
     <xsl:apply-templates select="." mode="create"/>
     <wbz:apply-templates select="."/>
    </wbz:view>
   </xsl:when>
   <xsl:otherwise>
     <wbz:apply-templates select="{$matched}"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <!-- sequnces doesn't requiere any special handling yet -->
 <xsl:template match="xsd:sequence">
  <xsl:if test="@maxOccurs&gt;1 or @maxOccurs='unbounded'">
    <xsl:message terminate="yes">Can't handle maxOccurs&gt;1 on sequence</xsl:message>
  </xsl:if>
  <xsl:if test="not(parent::node()/local-name()='complexType')">
    <xsl:message terminate="yes">Can't handle nested sequence</xsl:message>
  </xsl:if>
  <xsl:apply-templates/>
 </xsl:template>

 <!-- insert WbzRepeat if maxOccurs>0 or WbzChoice otherwise-->
 <xsl:template match="xsd:choice">
  <xsl:variable name="insertAt"> 
   <xsl:call-template name="insertPoint"/>
  </xsl:variable> 
  <xsl:variable name="matched"> 
   <xsl:for-each select="xsd:element">
    <xsl:if test="@maxOccurs&gt;1 or @maxOccurs='unbounded'">
      <xsl:message terminate="yes">Can't handle repeat inside a choice</xsl:message>
    </xsl:if>
    <xsl:if test="position()&gt;1">
     <xsl:value-of select="'|'"/>
    </xsl:if>
    <xsl:value-of select="concat(@ref,@name)"/>
   </xsl:for-each>
  </xsl:variable> 
  <xsl:choose>
   <xsl:when test="@maxOccurs&gt;1 or @maxOccurs='unbounded'">
    <wbz:view class="WbzRepeat" select="{$matched}" insertAt="{$insertAt}">
     <xsl:apply-templates mode="create"/>
     <wbz:apply-templates select="."/>
    </wbz:view>
   </xsl:when>
   <xsl:otherwise>
     <wbz:view class="WbzChoice" select="{$matched}">
      <xsl:apply-templates mode="create"/>
      <wbz:apply-templates select="."/>
     </wbz:view>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <!-- stylesheets in create mode construct a default element for each xsd:element-->
 <xsl:template match="xsd:element" mode="create">
   <xsl:variable name="name" select="concat(@ref,@name)"/> 
   <xsl:variable name="type" select="concat(@type,/xsd:schema/xsd:element[@name=$name]/@type)"/>
   <wbz:create>
    <xsl:attribute name="label" disable-output-escaping="yes">
     <xsl:if test="@name">
        <xsl:call-template name="entity-label"/>
     </xsl:if>
     <xsl:if test="@ref">
      <xsl:for-each select="/xsd:schema/xsd:element[@name=$name]">
       <xsl:call-template name="entity-label"/>
      </xsl:for-each>
     </xsl:if>
    </xsl:attribute>
    <xsl:element name="{$name}">
     <xsl:apply-templates select="/xsd:schema/xsd:complexType[@name=$type]" mode="create"/>
    </xsl:element>
   </wbz:create>
 </xsl:template>

 <xsl:template match="xsd:complexType" mode="create">
   <xsl:for-each select="xsd:attribute">
     <xsl:attribute name="{@name}"/>
   </xsl:for-each>
 </xsl:template>


</xsl:stylesheet>
  