<!DOCTYPE xsl:stylesheet
   [
   <!ENTITY annotationNS "http://wobzilla.sf.net/schema-annotation">
   <!ENTITY separator "$ENTITY_SEP$">
   <!ENTITY br "<xsl:text>
</xsl:text>">
   <!ENTITY myamp "<xsl:text disable-output-escaping='yes'>&amp;</xsl:text>">
   <!ENTITY mylt "<xsl:text disable-output-escaping='yes'>&lt;</xsl:text>">
   <!ENTITY mygt "<xsl:text disable-output-escaping='yes'>&gt;</xsl:text>">
   ]>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:doc="&annotationNS;"
    exclude-result-prefixes="doc xsd"
    version="1.0">

<xsl:param name="lang" select="'de'"/>

<!-- Generate entities for selected annotations 
Note: Annotations should not contain quotes "

-->
<xsl:template name="doctype">
   &br;
   &mylt;!DOCTYPE wbz:stylesheet [
<!-- External Entities don't work in mozilla and IE
       &mylt;!ENTITY % generatedEntities SYSTEM "demos/generated.ent" &mygt;
       %generatedEntities;
   <xsl:result-document href="generated.ent" format="entity-map">  	
-->   <xsl:for-each select="//xsd:element[@name]|//xsd:attribute">
        <xsl:variable name="entity">
         <xsl:call-template name="entity">
          <xsl:with-param name="name" select="'label'"/>
          <xsl:with-param name="namespace" select="'&annotationNS;'"/>
         </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="name" select="normalize-space(substring-before($entity,'&separator;'))"/>
        <xsl:variable name="value" select="normalize-space(substring-after($entity,'&separator;'))"/>
        <xsl:variable name="finalvalue">
         <xsl:choose>
          <xsl:when test="string-length($value) &gt; 0">
           <xsl:value-of select="$value"/>
          </xsl:when>
          <!-- If no annotation tag for the selected language is found, 
               use the element or attribute name! -->
          <xsl:otherwise>
           <xsl:value-of select="concat(@ref,@name)"/>
          </xsl:otherwise>
         </xsl:choose>
        </xsl:variable>
        &br;&mylt;!ENTITY <xsl:value-of select="$name"/> "<xsl:value-of select="$finalvalue"/>" &mygt;
      </xsl:for-each>
<!--
   </xsl:result-document>
--> 
   ]&mygt;
   &br;
 </xsl:template>


 <xsl:template name="entity-label">
   <xsl:variable name="entity">
     <xsl:call-template name="entityName">
      <xsl:with-param name="name" select="'label'"/>
      <xsl:with-param name="namespace" select="'&annotationNS;'"/>
     </xsl:call-template>
   </xsl:variable>
   <xsl:if test="string-length($entity)&gt;0">&myamp;<xsl:value-of select="$entity"/>;</xsl:if></xsl:template>

 <!-- returns the first part of the entity description entityName;entityValue -->
 <xsl:template name="entityName">
   <xsl:param name="name"/>
   <xsl:param name="namespace"/>
   <xsl:variable name="entity">
     <xsl:call-template name="entity">
      <xsl:with-param name="name" select="$name"/>
      <xsl:with-param name="namespace" select="$namespace"/>
     </xsl:call-template>
   </xsl:variable>
   <xsl:value-of select="substring-before($entity,'&separator;')"/></xsl:template>

 <!-- find for annotation with name = $name and namespace = $namespace 
and xml:lang = $lang 
-->
 <xsl:template name="entity">
   <xsl:param name="name"/>
   <xsl:param name="namespace"/>
   <!-- Look for the annotation elements in attributes or in direct childs
           #TODO Should also look in xsd:annotation
    -->
   <xsl:variable name="value" 
    select="concat(*[namespace-uri()=$namespace and local-name()=$name and (not(@xml:lang) or @xml:lang=$lang)],@*[namespace-uri()=$namespace and local-name()=$name])" 
    />

   <xsl:variable name="nodeName">
    <xsl:choose>
     <xsl:when test="local-name()='attribute'">
       <xsl:value-of select=
         "concat(ancestor::xsd:complexType[@name]/@name,ancestor::xsd:element[@name]/name,'.',@name)"
        />
     </xsl:when>
     <xsl:when test="local-name()='element'">
       <xsl:value-of select="@name"/>
     </xsl:when>
     <xsl:otherwise>
     </xsl:otherwise>
    </xsl:choose>
   </xsl:variable>
   <xsl:value-of select="normalize-space(concat($nodeName,'.',$name,'&separator;',$value))"/>
 </xsl:template>
</xsl:stylesheet>